using System;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Security.Cryptography;
using System.Runtime.InteropServices;
using System.Globalization;
using ShellLib;

[assembly: System.Reflection.AssemblyKeyFile(@"..\..\KeyFile.snk")]

namespace ThunderMain.Shell 
{   
    /// <summary>
    /// Summary description for DeepColumnHandler.
    /// 
    /// Needs to be registered with RegAsm.exe and installed in the gac with GacUtil.exe
    /// </summary>
    [Guid("8357160C-7333-45d5-9186-B33EB28D3EA1"), ComVisible(true)]
    public class DeepColumnHandler : ColumnProvider 
    {
        const int S_OK=0;
        const int S_FALSE=1;

        public DeepColumnHandler()
        {
        }

        public override int Initialize(LPCSHCOLUMNINIT psci) 
        {
            return S_OK;
        }

        public override int GetColumnInfo(int dwIndex, out SHCOLUMNINFO psci) 
        {           
            psci=new SHCOLUMNINFO();
            
            if(dwIndex!=0) 
                return S_FALSE;

            try 
            {
                psci.scid.fmtid=GetType().GUID;         
                psci.scid.pid=0;

                // Cast to a ushort, because a VARTYPE is ushort and a VARENUM is int
                psci.vt=(ushort)VarEnum.VT_I8;
                psci.fmt=LVCFMT.RIGHT;
                psci.cChars=40;

                psci.csFlags=SHCOLSTATE.TYPE_INT | SHCOLSTATE.ONBYDEFAULT; // | SHCOLSTATE.SLOW;

                psci.wszTitle = "Deep Size";
                psci.wszDescription = "Provides a recursive size of t he directory, in Kilobytes";
            } 
            catch(Exception e) 
            {
                MessageBox.Show(e.Message);
                return S_FALSE;
            }

            return S_OK;
        }

        public override int GetItemData( LPCSHCOLUMNID pscid, LPCSHCOLUMNDATA pscd, out object pvarData) 
        {
            pvarData=string.Empty;

            // Only service known columns
            if(pscid.fmtid!=GetType().GUID || pscid.pid!=0)
                return S_FALSE;

            try 
            { 
                DirectoryInfo dirinfo = new DirectoryInfo(pscd.wszFile);
				
				String uprpath = dirinfo.ToString().ToUpper();

				if (uprpath.EndsWith("PROGRAM FILES") ||
					uprpath.EndsWith("DOCUMENTS AND SETTINGS") ||
					uprpath.EndsWith("WINDOWS") ||
					uprpath.EndsWith("WIN98") ||
					uprpath.EndsWith("WINME") ||
					uprpath.EndsWith("I386") ||
					uprpath.EndsWith("WINNT"))
				{
					pvarData = "Skipped";
				}
				else
				{
					long size = getDirectoryDeepSize(dirinfo);
					pvarData = deepSizeInKiloBytes(size);
				}
            } 
            catch(UnauthorizedAccessException /*e*/) 
            {
                return S_FALSE;
            }
            catch (PathTooLongException e)
            {
                pvarData = "Path is too deep (" + e.Message + ")";
            }
            catch(Exception e) 
            {
                pvarData = e.Message;
                return S_FALSE;
            }

            return S_OK;
        }

        long getDirectoryDeepSize(DirectoryInfo dir)
        {
            long size = 0;

            // Add file sizes.
            FileInfo[] files = dir.GetFiles();
            foreach (FileInfo f in files) 
            {      
                size += f.Length;    
            }

            // Add subdirectory sizes.
            DirectoryInfo[] subdirs = dir.GetDirectories();
            foreach (DirectoryInfo d in subdirs) 
            {
                size += getDirectoryDeepSize(d);   
            }

            return size;
        }

        String deepSizeInKiloBytes(long size)
        {
            long k = 1024;

            if (size != 0)
            {
                size /= k;

                if (size == 0)
                {
                    size = 1;
                }
            }

//            NumberFormatInfo nfi = new CultureInfo( "en-US", false ).NumberFormat;
//            nfi.NumberDecimalDigits = 0;
//            return size.ToString("N", nfi);
            return size.ToString() + " KB";
        }
    }
}
